@@ -1,6 +1,8 @@
 from re import findall
+from json import dumps as json_dumps
 from gitlab_ps_utils.api import GitLabApi
-from gitlab_ps_utils.misc_utils import safe_json_response
+from gitlab_ps_utils.misc_utils import safe_json_response, is_error_message_present
+from gitlab_ps_utils.dict_utils import dig
 from gitlab_evaluate.lib import human_bytes as hb
 from gitlab_evaluate.lib import utils
 
@@ -19,41 +21,6 @@ def proj_info_get(i, source):
     '''Trying to create proper api call with project id.'''
     return f"{source}/api/v4/projects/{str(i)}?statistics=true"
 
-def proj_pl_get(i, source):
-    '''Trying to create proper api call with project id to get the number of pipelines.'''
-    i = str(i)
-    pl_inf_url = f"/api/v4/projects/{i}/pipelines"
-    gl_proj_pl = source + pl_inf_url
-    return gl_proj_pl
-
-def proj_issue_get(i, source):
-    '''Trying to create proper api call with project id to get the number of issues.'''
-    i = str(i)
-    issue_inf_url = f"/api/v4/projects/{i}/issues"
-    gl_issue_pl = source + issue_inf_url
-    return gl_issue_pl
-
-def proj_branch_get(i, source):
-    '''Trying to create proper api call with project id to get the number of branches.'''
-    i = str(i)
-    branch_inf_url = f"/api/v4/projects/{i}/repository/branches"
-    gl_branch_pl = source + branch_inf_url
-    return gl_branch_pl
-
-def proj_mr_get(i, source):
-    '''Trying to create proper api call with project id to get the number of MR's.'''
-    i = str(i)
-    mr_inf_url = f"/api/v4/projects/{i}/merge_requests"
-    gl_mr_pl = source + mr_inf_url
-    return gl_mr_pl
-
-def proj_tag_get(i, source):
-    '''Trying to create proper api call with project id to get the number of tags.'''
-    i = str(i)
-    tag_inf_url = f"/api/v4/projects/{i}/repository/tags"
-    gl_tag_pl = source + tag_inf_url
-    return gl_tag_pl
-
 def proj_packages_url(i):
     return f"projects/{str(i)}/packages"
 
@@ -77,24 +44,44 @@ def get_registry_details(i):
 #     return gl_commits_pl       
 ### Functions - Return API Data
 # Gets the X-Total from the statistics page with the -I on a curl
-def check_x_total_value_update_dict(check_func, p, url="", headers={}, value_column_name="DEFAULT_VALUE", over_column_name="DEFAULT_COLUMN_NAME", results={}):
+def check_x_total_value_update_dict(check_func, p, host, token, api, headers={}, value_column_name="DEFAULT_VALUE", over_column_name="DEFAULT_COLUMN_NAME", results={}):
     flag = False
-    if resp := gl_api.generate_get_request(host="", api="", token=headers.get("PRIVATE-TOKEN"), url=url): 
-        num = int(resp.headers.get('X-Total', 0))
-        num_over = check_func(num)
+    count = get_total_count(host, token, api, p['path_with_namespace'], value_column_name)
+    if count:
+        num_over = check_func(count)
         if num_over:
             flag = True
-        results[value_column_name] = num
+        results[value_column_name] = count
         results[over_column_name] = num_over
     else:
-        print(f"Could not retrieve {value_column_name} for project: {p.get('id')} - {p.get('name')}")
+        print(f"Could not retrieve {value_column_name} for project: {p.get('id')} - {p.get('path_with_namespace')}")
     return flag
-        
+
+def get_total_count(host, token, api, full_path, entity):
+    if count := gl_api.get_count(host, token, api):
+        return count
+    else:
+        formatted_entity = utils.to_camel_case(entity)
+        query = {
+            "query": """
+                query {
+                    project(fullPath: "%s") {
+                        name,
+                        %s {
+                            count
+                        }
+                    }
+                }
+            """ % (full_path, formatted_entity)
+        }
+
+        if gql_resp := safe_json_response(gl_api.generate_post_request(host, token, None, json_dumps(query), graphql_query=True)):
+            return dig(gql_resp, 'data', 'project', formatted_entity, 'count')
+
+
 # gets the full stats of the project and sorts based on the returned items, passing a few through the HumanReadable utility
 def check_full_stats(url, project, my_dict, headers={}):
-    if resp := gl_api.generate_get_request(host="", api="", token=headers.get("PRIVATE-TOKEN"), url=url): 
-        
-        result = resp.json()
+    if result := safe_json_response(gl_api.generate_get_request(host="", api="", token=headers.get("PRIVATE-TOKEN"), url=url)): 
         if kind := result.get("namespace"):
             my_dict.update({"kind": kind.get("kind")})
         if stats := result.get("statistics"):
@@ -108,7 +95,8 @@ def check_full_stats(url, project, my_dict, headers={}):
             # packages_size = stats.get['packages_size']
             export_total = 0
             for k, v in stats.items():
-                my_dict.update({ k: hb.HumanBytes.format(v, True) if k != "commit_count" else v, k + "_over": utils.check_size(k, v)}) 
+                updated_dict_entry = { k: hb.HumanBytes.format(v, True) if k != "commit_count" else v, k + "_over": utils.check_size(k, v)}
+                my_dict.update(updated_dict_entry)
 
                 # If k an item that would be part of the export, add to running total
                 if k in [
@@ -130,31 +118,56 @@ def check_full_stats(url, project, my_dict, headers={}):
             export_total = 0
             # reset running total? Not loop, so maybe not
         else:
-            print(f"Could not extracts stats for project {project.get('name')}.\n")
+            print(f"Could not extracts stats for project {project.get('path_with_namepsace')}.\n")
     else:
         print(f"Could not retrieve project with stats for project id: {project.get('id')}")
 
-def get_registry_size(pid, source, token):
+def get_registry_size(pid, path_with_namespace, source, token):
     """
         Iterates over a project's registry data and returns the total size of registry data
     """
-    registry_hashes = {}
-    for registry_repo in gl_api.list_all(source, token, proj_registries_url(pid)):
-        if isinstance(registry_repo, str):
-            print(' Container registry inaccessible or disabled. Skipping')
-            continue
-        rid = registry_repo['id']
-        for registry_tags in gl_api.list_all(source, token, proj_registries_tags_url(pid, rid)):
-            # print(registry_tags)
-            tid = registry_tags['name']
-            repo_details = safe_json_response(gl_api.generate_get_request(source, token, proj_registries_tag_details_url(pid, rid, tid)))
-            if repo_size := repo_details.get('total_size'):
-                registry_hashes['short_revision'] = repo_size
+    if total_size := get_registry_size_by_graphql(path_with_namespace, source, token):
+        print('Retrieved registry size by single GraphQL query')
+    else:
+        print('Failed to receive registry size by GraphQL queries. Falling back to REST calls')
+        registry_hashes = {}
+        for registry_repo in gl_api.list_all(source, token, proj_registries_url(pid)):
+            error, _ = is_error_message_present(registry_repo)
+            if error:
+                print(f'Container registry from {path_with_namespace} inaccessible or disabled. Skipping')
+                continue
+            rid = registry_repo['id']
+            for registry_tags in gl_api.list_all(source, token, proj_registries_tags_url(pid, rid)):
+                tid = registry_tags['name']
+                repo_details = safe_json_response(gl_api.generate_get_request(source, token, proj_registries_tag_details_url(pid, rid, tid)))
+                if repo_size := repo_details.get('total_size'):
+                    registry_hashes['short_revision'] = repo_size
 
-    total_size = sum(registry_hashes.values())
+        total_size = sum(registry_hashes.values())
 
     return utils.sizeof_fmt(total_size), utils.check_storage_size(total_size)
 
+def get_registry_size_by_graphql(path_with_namespace, source, token):
+    """
+        Makes a single GraphQL query to get the total registry size
+        of a project. This query could return an error stating
+        the query result is too large
+    """
+    query = {
+        'query': """
+            query {
+                project(fullPath: "%s") {
+                    statistics {
+                        containerRegistrySize
+                        }
+                    }
+                }
+                
+        """ % path_with_namespace
+    }
+    if gql_resp := safe_json_response(gl_api.generate_post_request(source, token, None, data=json_dumps(query), graphql_query=True)):
+        return dig(gql_resp, 'data', 'project', 'statistics', 'containerRegistrySize')
+
 def getApplicationInfo(host,token,api):
     if resp := gl_api.generate_get_request(host=host, token=token, api=api):
         result = resp.json()
