@@ -1,5 +1,6 @@
 import csv
 import sys
+from re import sub
 from gitlab_evaluate.lib import limits
 
 
@@ -73,3 +74,10 @@ def sizeof_fmt(num, suffix="B"):
             return f"{num:3.1f}{unit}{suffix}"
         num /= 1024.0
     return f"{num:.1f}Yi{suffix}"
+
+"""
+    shameless copy from https://www.w3resource.com/python-exercises/string/python-data-type-string-exercise-96.php
+"""
+def to_camel_case(s):
+    s = sub(r"(_|-)+", " ", s).title().replace(" ", "")
+    return ''.join([s[0].lower(), s[1:]])
